from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from sentinos_core import AuthenticatedClient, Client

from ._http import request_json, request_json_async, require_tenant


def _items(body: dict[str, Any], *keys: str) -> list[dict[str, Any]]:
    for key in keys:
        value = body.get(key)
        if isinstance(value, list):
            return list(value)
    return []


@dataclass
class ControlplaneClient:
    _core: Client | AuthenticatedClient
    tenant_id: str | None = None

    def _require_org(self, org_id: str | None) -> str:
        return require_tenant(self.tenant_id, org_id, field_name="org_id")

    def _request(
        self,
        method: str,
        path: str,
        *,
        org_id: str | None = None,
        tenant_required: bool = False,
        params: dict[str, Any] | None = None,
        body: Any = None,
    ) -> dict[str, Any]:
        tenant_id = self._require_org(org_id) if tenant_required else (org_id or self.tenant_id)
        return request_json(self._core, method, path, tenant_id=tenant_id, params=params, body=body)

    async def _request_async(
        self,
        method: str,
        path: str,
        *,
        org_id: str | None = None,
        tenant_required: bool = False,
        params: dict[str, Any] | None = None,
        body: Any = None,
    ) -> dict[str, Any]:
        tenant_id = self._require_org(org_id) if tenant_required else (org_id or self.tenant_id)
        return await request_json_async(self._core, method, path, tenant_id=tenant_id, params=params, body=body)

    def list_organizations(self) -> list[dict[str, Any]]:
        body = self._request("GET", "/v1/orgs")
        return _items(body, "organizations", "orgs")

    async def list_organizations_async(self) -> list[dict[str, Any]]:
        body = await self._request_async("GET", "/v1/orgs")
        return _items(body, "organizations", "orgs")

    def create_organization(self, *, organization: dict[str, Any]) -> dict[str, Any]:
        return self._request("POST", "/v1/orgs", body=organization)

    async def create_organization_async(self, *, organization: dict[str, Any]) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/orgs", body=organization)

    def get_organization(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}", org_id=oid, tenant_required=True)

    async def get_organization_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async("GET", f"/v1/orgs/{oid}", org_id=oid, tenant_required=True)

    def patch_organization(self, *, organization: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("PATCH", f"/v1/orgs/{oid}", org_id=oid, tenant_required=True, body=organization)

    async def patch_organization_async(
        self, *, organization: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}",
            org_id=oid,
            tenant_required=True,
            body=organization,
        )

    def switch_context_token(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/switch-context-token", org_id=oid, tenant_required=True)

    async def switch_context_token_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/switch-context-token",
            org_id=oid,
            tenant_required=True,
        )

    def register(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return self._request("POST", "/v1/auth/register", body=payload)

    async def register_async(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/auth/register", body=payload)

    def login_password(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return self._request("POST", "/v1/auth/login/password", body=payload)

    async def login_password_async(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/auth/login/password", body=payload)

    def refresh_token(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return self._request("POST", "/v1/auth/token/refresh", body=payload)

    async def refresh_token_async(self, *, payload: dict[str, Any]) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/auth/token/refresh", body=payload)

    def auth_me(self) -> dict[str, Any]:
        return self._request("GET", "/v1/auth/me")

    async def auth_me_async(self) -> dict[str, Any]:
        return await self._request_async("GET", "/v1/auth/me")

    def logout(self, *, payload: dict[str, Any] | None = None) -> dict[str, Any]:
        return self._request("POST", "/v1/auth/logout", body=payload)

    async def logout_async(self, *, payload: dict[str, Any] | None = None) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/auth/logout", body=payload)

    def list_sessions(self) -> list[dict[str, Any]]:
        body = self._request("GET", "/v1/auth/sessions")
        return _items(body, "sessions", "items")

    async def list_sessions_async(self) -> list[dict[str, Any]]:
        body = await self._request_async("GET", "/v1/auth/sessions")
        return _items(body, "sessions", "items")

    def revoke_other_sessions(self) -> dict[str, Any]:
        return self._request("POST", "/v1/auth/sessions/revoke-others")

    async def revoke_other_sessions_async(self) -> dict[str, Any]:
        return await self._request_async("POST", "/v1/auth/sessions/revoke-others")

    def delete_session(self, session_id: str) -> dict[str, Any]:
        return self._request("DELETE", f"/v1/auth/sessions/{session_id}")

    async def delete_session_async(self, session_id: str) -> dict[str, Any]:
        return await self._request_async("DELETE", f"/v1/auth/sessions/{session_id}")

    def list_members(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/members", org_id=oid, tenant_required=True)
        return _items(body, "members", "items")

    async def list_members_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/members", org_id=oid, tenant_required=True)
        return _items(body, "members", "items")

    def patch_member(
        self, membership_id: str, *, membership: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
            body=membership,
        )

    async def patch_member_async(
        self, membership_id: str, *, membership: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
            body=membership,
        )

    def remove_member(self, membership_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("DELETE", f"/v1/orgs/{oid}/members/{membership_id}", org_id=oid, tenant_required=True)

    async def remove_member_async(self, membership_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    def list_invites(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/invites", org_id=oid, tenant_required=True)
        return _items(body, "invites", "invitations", "items")

    async def list_invites_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/invites", org_id=oid, tenant_required=True)
        return _items(body, "invites", "invitations", "items")

    def create_invite(self, *, invite: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/invites", org_id=oid, tenant_required=True, body=invite)

    async def create_invite_async(self, *, invite: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/invites",
            org_id=oid,
            tenant_required=True,
            body=invite,
        )

    def get_invite(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/invites/{invitation_id}", org_id=oid, tenant_required=True)

    async def get_invite_async(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/invites/{invitation_id}",
            org_id=oid,
            tenant_required=True,
        )

    def resend_invite(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/invites/{invitation_id}/resend",
            org_id=oid,
            tenant_required=True,
        )

    async def resend_invite_async(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/invites/{invitation_id}/resend",
            org_id=oid,
            tenant_required=True,
        )

    def cancel_invite(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/invites/{invitation_id}/cancel",
            org_id=oid,
            tenant_required=True,
        )

    async def cancel_invite_async(self, invitation_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/invites/{invitation_id}/cancel",
            org_id=oid,
            tenant_required=True,
        )

    def list_permissions(self) -> list[dict[str, Any]]:
        body = self._request("GET", "/v1/permissions")
        return _items(body, "permissions", "items")

    async def list_permissions_async(self) -> list[dict[str, Any]]:
        body = await self._request_async("GET", "/v1/permissions")
        return _items(body, "permissions", "items")

    def list_roles(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/roles", org_id=oid, tenant_required=True)
        return _items(body, "roles", "items")

    async def list_roles_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/roles", org_id=oid, tenant_required=True)
        return _items(body, "roles", "items")

    def create_role(self, *, role: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/roles", org_id=oid, tenant_required=True, body=role)

    async def create_role_async(self, *, role: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/roles",
            org_id=oid,
            tenant_required=True,
            body=role,
        )

    def patch_role(self, role_id: str, *, role: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/roles/{role_id}",
            org_id=oid,
            tenant_required=True,
            body=role,
        )

    async def patch_role_async(
        self, role_id: str, *, role: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/roles/{role_id}",
            org_id=oid,
            tenant_required=True,
            body=role,
        )

    def delete_role(self, role_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("DELETE", f"/v1/orgs/{oid}/roles/{role_id}", org_id=oid, tenant_required=True)

    async def delete_role_async(self, role_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/roles/{role_id}",
            org_id=oid,
            tenant_required=True,
        )

    def get_role_permissions(self, role_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "GET",
            f"/v1/orgs/{oid}/roles/{role_id}/permissions",
            org_id=oid,
            tenant_required=True,
        )

    async def get_role_permissions_async(self, role_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/roles/{role_id}/permissions",
            org_id=oid,
            tenant_required=True,
        )

    def put_role_permissions(
        self, role_id: str, *, permissions: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PUT",
            f"/v1/orgs/{oid}/roles/{role_id}/permissions",
            org_id=oid,
            tenant_required=True,
            body=permissions,
        )

    async def put_role_permissions_async(
        self, role_id: str, *, permissions: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PUT",
            f"/v1/orgs/{oid}/roles/{role_id}/permissions",
            org_id=oid,
            tenant_required=True,
            body=permissions,
        )

    def assign_role_to_member(
        self, role_id: str, membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/roles/{role_id}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def assign_role_to_member_async(
        self, role_id: str, membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/roles/{role_id}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    def unassign_role_from_member(
        self, role_id: str, membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/roles/{role_id}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def unassign_role_from_member_async(
        self, role_id: str, membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/roles/{role_id}/members/{membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    def list_teams(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/teams", org_id=oid, tenant_required=True)
        return _items(body, "teams", "items")

    async def list_teams_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/teams", org_id=oid, tenant_required=True)
        return _items(body, "teams", "items")

    def create_team(self, *, team: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/teams", org_id=oid, tenant_required=True, body=team)

    async def create_team_async(self, *, team: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/teams",
            org_id=oid,
            tenant_required=True,
            body=team,
        )

    def patch_team(self, team_id: str, *, team: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/teams/{team_id}",
            org_id=oid,
            tenant_required=True,
            body=team,
        )

    async def patch_team_async(
        self, team_id: str, *, team: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/teams/{team_id}",
            org_id=oid,
            tenant_required=True,
            body=team,
        )

    def delete_team(self, team_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("DELETE", f"/v1/orgs/{oid}/teams/{team_id}", org_id=oid, tenant_required=True)

    async def delete_team_async(self, team_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/teams/{team_id}",
            org_id=oid,
            tenant_required=True,
        )

    def list_team_memberships(self, team_id: str, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request(
            "GET",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships",
            org_id=oid,
            tenant_required=True,
        )
        return _items(body, "memberships", "items")

    async def list_team_memberships_async(self, team_id: str, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships",
            org_id=oid,
            tenant_required=True,
        )
        return _items(body, "memberships", "items")

    def create_team_membership(
        self, team_id: str, *, membership: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships",
            org_id=oid,
            tenant_required=True,
            body=membership,
        )

    async def create_team_membership_async(
        self, team_id: str, *, membership: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships",
            org_id=oid,
            tenant_required=True,
            body=membership,
        )

    def delete_team_membership(
        self, team_id: str, team_membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships/{team_membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def delete_team_membership_async(
        self, team_id: str, team_membership_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/teams/{team_id}/memberships/{team_membership_id}",
            org_id=oid,
            tenant_required=True,
        )

    def get_team_settings(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/teams/settings", org_id=oid, tenant_required=True)

    async def get_team_settings_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async("GET", f"/v1/orgs/{oid}/teams/settings", org_id=oid, tenant_required=True)

    def patch_team_settings(self, *, settings: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/teams/settings",
            org_id=oid,
            tenant_required=True,
            body=settings,
        )

    async def patch_team_settings_async(
        self, *, settings: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/teams/settings",
            org_id=oid,
            tenant_required=True,
            body=settings,
        )

    def list_service_accounts(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/service-accounts", org_id=oid, tenant_required=True)
        return _items(body, "service_accounts", "accounts", "items")

    async def list_service_accounts_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/service-accounts", org_id=oid, tenant_required=True)
        return _items(body, "service_accounts", "accounts", "items")

    def create_service_account(self, *, account: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/service-accounts",
            org_id=oid,
            tenant_required=True,
            body=account,
        )

    async def create_service_account_async(
        self, *, account: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/service-accounts",
            org_id=oid,
            tenant_required=True,
            body=account,
        )

    def patch_service_account(
        self, service_account_id: str, *, account: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}",
            org_id=oid,
            tenant_required=True,
            body=account,
        )

    async def patch_service_account_async(
        self, service_account_id: str, *, account: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}",
            org_id=oid,
            tenant_required=True,
            body=account,
        )

    def create_service_account_token(
        self, service_account_id: str, *, token: dict[str, Any] | None = None, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}/tokens",
            org_id=oid,
            tenant_required=True,
            body=token,
        )

    async def create_service_account_token_async(
        self, service_account_id: str, *, token: dict[str, Any] | None = None, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}/tokens",
            org_id=oid,
            tenant_required=True,
            body=token,
        )

    def revoke_service_account_token(
        self, service_account_id: str, token_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}/tokens/{token_id}/revoke",
            org_id=oid,
            tenant_required=True,
        )

    async def revoke_service_account_token_async(
        self, service_account_id: str, token_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/service-accounts/{service_account_id}/tokens/{token_id}/revoke",
            org_id=oid,
            tenant_required=True,
        )

    def get_login_methods(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/settings/login-methods", org_id=oid, tenant_required=True)

    async def get_login_methods_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/settings/login-methods",
            org_id=oid,
            tenant_required=True,
        )

    def patch_login_methods(self, *, settings: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/settings/login-methods",
            org_id=oid,
            tenant_required=True,
            body=settings,
        )

    async def patch_login_methods_async(
        self, *, settings: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/settings/login-methods",
            org_id=oid,
            tenant_required=True,
            body=settings,
        )

    def get_user_login_method_override(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "GET",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
        )

    async def get_user_login_method_override_async(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
        )

    def patch_user_login_method_override(
        self, user_id: str, *, override: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
            body=override,
        )

    async def patch_user_login_method_override_async(
        self, user_id: str, *, override: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
            body=override,
        )

    def delete_user_login_method_override(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
        )

    async def delete_user_login_method_override_async(
        self, user_id: str, *, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/users/{user_id}/login-method-override",
            org_id=oid,
            tenant_required=True,
        )

    def get_saml_config(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/saml/config", org_id=oid, tenant_required=True)

    async def get_saml_config_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async("GET", f"/v1/orgs/{oid}/saml/config", org_id=oid, tenant_required=True)

    def patch_saml_config(self, *, config: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("PATCH", f"/v1/orgs/{oid}/saml/config", org_id=oid, tenant_required=True, body=config)

    async def patch_saml_config_async(self, *, config: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/saml/config",
            org_id=oid,
            tenant_required=True,
            body=config,
        )

    def get_saml_metadata(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/saml/metadata", org_id=oid, tenant_required=True)

    async def get_saml_metadata_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async("GET", f"/v1/orgs/{oid}/saml/metadata", org_id=oid, tenant_required=True)

    def post_saml_acs(self, *, payload: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/saml/acs", org_id=oid, tenant_required=True, body=payload)

    async def post_saml_acs_async(self, *, payload: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/saml/acs",
            org_id=oid,
            tenant_required=True,
            body=payload,
        )

    def list_authn_mappings(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/authn-mappings", org_id=oid, tenant_required=True)
        return _items(body, "mappings", "items")

    async def list_authn_mappings_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async("GET", f"/v1/orgs/{oid}/authn-mappings", org_id=oid, tenant_required=True)
        return _items(body, "mappings", "items")

    def create_authn_mapping(self, *, mapping: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/authn-mappings",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    async def create_authn_mapping_async(
        self, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/authn-mappings",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    def patch_authn_mapping(
        self, mapping_id: str, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/authn-mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    async def patch_authn_mapping_async(
        self, mapping_id: str, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/authn-mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    def delete_authn_mapping(self, mapping_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/authn-mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def delete_authn_mapping_async(self, mapping_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/authn-mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
        )

    def get_scim_config(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/scim/config", org_id=oid, tenant_required=True)

    async def get_scim_config_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async("GET", f"/v1/orgs/{oid}/scim/config", org_id=oid, tenant_required=True)

    def create_scim_token(self, *, token: dict[str, Any] | None = None, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("POST", f"/v1/orgs/{oid}/scim/tokens", org_id=oid, tenant_required=True, body=token)

    async def create_scim_token_async(
        self, *, token: dict[str, Any] | None = None, org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/scim/tokens",
            org_id=oid,
            tenant_required=True,
            body=token,
        )

    def delete_scim_token(self, token_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/scim/tokens/{token_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def delete_scim_token_async(self, token_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/scim/tokens/{token_id}",
            org_id=oid,
            tenant_required=True,
        )

    def scim_list_users(
        self,
        *,
        start_index: int | None = None,
        count: int | None = None,
        filter: str | None = None,
        org_id: str | None = None,
    ) -> dict[str, Any]:
        params = {"startIndex": start_index, "count": count, "filter": filter}
        return self._request("GET", "/scim/v2/Users", org_id=org_id, params=params)

    async def scim_list_users_async(
        self,
        *,
        start_index: int | None = None,
        count: int | None = None,
        filter: str | None = None,
        org_id: str | None = None,
    ) -> dict[str, Any]:
        params = {"startIndex": start_index, "count": count, "filter": filter}
        return await self._request_async("GET", "/scim/v2/Users", org_id=org_id, params=params)

    def scim_create_user(self, *, user: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        return self._request("POST", "/scim/v2/Users", org_id=org_id, body=user)

    async def scim_create_user_async(self, *, user: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        return await self._request_async("POST", "/scim/v2/Users", org_id=org_id, body=user)

    def scim_get_user(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return self._request("GET", f"/scim/v2/Users/{user_id}", org_id=org_id)

    async def scim_get_user_async(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return await self._request_async("GET", f"/scim/v2/Users/{user_id}", org_id=org_id)

    def scim_patch_user(self, user_id: str, *, patch: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        return self._request("PATCH", f"/scim/v2/Users/{user_id}", org_id=org_id, body=patch)

    async def scim_patch_user_async(
        self, user_id: str, *, patch: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        return await self._request_async("PATCH", f"/scim/v2/Users/{user_id}", org_id=org_id, body=patch)

    def scim_delete_user(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return self._request("DELETE", f"/scim/v2/Users/{user_id}", org_id=org_id)

    async def scim_delete_user_async(self, user_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return await self._request_async("DELETE", f"/scim/v2/Users/{user_id}", org_id=org_id)

    def scim_list_groups(
        self,
        *,
        start_index: int | None = None,
        count: int | None = None,
        filter: str | None = None,
        org_id: str | None = None,
    ) -> dict[str, Any]:
        params = {"startIndex": start_index, "count": count, "filter": filter}
        return self._request("GET", "/scim/v2/Groups", org_id=org_id, params=params)

    async def scim_list_groups_async(
        self,
        *,
        start_index: int | None = None,
        count: int | None = None,
        filter: str | None = None,
        org_id: str | None = None,
    ) -> dict[str, Any]:
        params = {"startIndex": start_index, "count": count, "filter": filter}
        return await self._request_async("GET", "/scim/v2/Groups", org_id=org_id, params=params)

    def scim_create_group(self, *, group: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        return self._request("POST", "/scim/v2/Groups", org_id=org_id, body=group)

    async def scim_create_group_async(self, *, group: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        return await self._request_async("POST", "/scim/v2/Groups", org_id=org_id, body=group)

    def scim_patch_group(
        self, group_id: str, *, patch: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        return self._request("PATCH", f"/scim/v2/Groups/{group_id}", org_id=org_id, body=patch)

    async def scim_patch_group_async(
        self, group_id: str, *, patch: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        return await self._request_async("PATCH", f"/scim/v2/Groups/{group_id}", org_id=org_id, body=patch)

    def scim_delete_group(self, group_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return self._request("DELETE", f"/scim/v2/Groups/{group_id}", org_id=org_id)

    async def scim_delete_group_async(self, group_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        return await self._request_async("DELETE", f"/scim/v2/Groups/{group_id}", org_id=org_id)

    def get_workforce_policy(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/workforce/policy", org_id=oid, tenant_required=True)

    async def get_workforce_policy_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/policy",
            org_id=oid,
            tenant_required=True,
        )

    def update_workforce_policy(self, *, policy: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/policy",
            org_id=oid,
            tenant_required=True,
            body=policy,
        )

    async def update_workforce_policy_async(
        self, *, policy: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/policy",
            org_id=oid,
            tenant_required=True,
            body=policy,
        )

    def list_workforce_mappings(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request("GET", f"/v1/orgs/{oid}/workforce/mappings", org_id=oid, tenant_required=True)
        return _items(body, "mappings", "items")

    async def list_workforce_mappings_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/mappings",
            org_id=oid,
            tenant_required=True,
        )
        return _items(body, "mappings", "items")

    def create_workforce_mapping(self, *, mapping: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/workforce/mappings",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    async def create_workforce_mapping_async(
        self, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/workforce/mappings",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    def update_workforce_mapping(
        self, mapping_id: str, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    async def update_workforce_mapping_async(
        self, mapping_id: str, *, mapping: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
            body=mapping,
        )

    def delete_workforce_mapping(self, mapping_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "DELETE",
            f"/v1/orgs/{oid}/workforce/mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
        )

    async def delete_workforce_mapping_async(self, mapping_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "DELETE",
            f"/v1/orgs/{oid}/workforce/mappings/{mapping_id}",
            org_id=oid,
            tenant_required=True,
        )

    def list_workforce_subjects(self, *, org_id: str | None = None, limit: int | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request(
            "GET",
            f"/v1/orgs/{oid}/workforce/subjects",
            org_id=oid,
            tenant_required=True,
            params={"limit": limit},
        )
        return _items(body, "subjects", "items")

    async def list_workforce_subjects_async(
        self, *, org_id: str | None = None, limit: int | None = None
    ) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/subjects",
            org_id=oid,
            tenant_required=True,
            params={"limit": limit},
        )
        return _items(body, "subjects", "items")

    def list_workforce_sessions(self, *, org_id: str | None = None, limit: int | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request(
            "GET",
            f"/v1/orgs/{oid}/workforce/sessions",
            org_id=oid,
            tenant_required=True,
            params={"limit": limit},
        )
        return _items(body, "sessions", "items")

    async def list_workforce_sessions_async(
        self, *, org_id: str | None = None, limit: int | None = None
    ) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/sessions",
            org_id=oid,
            tenant_required=True,
            params={"limit": limit},
        )
        return _items(body, "sessions", "items")

    def revoke_workforce_session(self, session_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/workforce/sessions/{session_id}/revoke",
            org_id=oid,
            tenant_required=True,
        )

    async def revoke_workforce_session_async(self, session_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/workforce/sessions/{session_id}/revoke",
            org_id=oid,
            tenant_required=True,
        )

    def list_workforce_audit(
        self,
        *,
        org_id: str | None = None,
        actor: str | None = None,
        action: str | None = None,
        limit: int | None = None,
        cursor: str | None = None,
    ) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = self._request(
            "GET",
            f"/v1/orgs/{oid}/workforce/audit",
            org_id=oid,
            tenant_required=True,
            params={"actor": actor, "action": action, "limit": limit, "cursor": cursor},
        )
        return _items(body, "events", "audit", "items")

    async def list_workforce_audit_async(
        self,
        *,
        org_id: str | None = None,
        actor: str | None = None,
        action: str | None = None,
        limit: int | None = None,
        cursor: str | None = None,
    ) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/audit",
            org_id=oid,
            tenant_required=True,
            params={"actor": actor, "action": action, "limit": limit, "cursor": cursor},
        )
        return _items(body, "events", "audit", "items")

    def create_workforce_rollout_wave(self, *, wave: dict[str, Any], org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/workforce/rollout/waves",
            org_id=oid,
            tenant_required=True,
            body=wave,
        )

    async def create_workforce_rollout_wave_async(
        self, *, wave: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/workforce/rollout/waves",
            org_id=oid,
            tenant_required=True,
            body=wave,
        )

    def update_workforce_rollout_wave(
        self, wave_id: str, *, wave: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/rollout/waves/{wave_id}",
            org_id=oid,
            tenant_required=True,
            body=wave,
        )

    async def update_workforce_rollout_wave_async(
        self, wave_id: str, *, wave: dict[str, Any], org_id: str | None = None
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "PATCH",
            f"/v1/orgs/{oid}/workforce/rollout/waves/{wave_id}",
            org_id=oid,
            tenant_required=True,
            body=wave,
        )

    def rollback_workforce_rollout_wave(self, wave_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request(
            "POST",
            f"/v1/orgs/{oid}/workforce/rollout/waves/{wave_id}/rollback",
            org_id=oid,
            tenant_required=True,
        )

    async def rollback_workforce_rollout_wave_async(self, wave_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "POST",
            f"/v1/orgs/{oid}/workforce/rollout/waves/{wave_id}/rollback",
            org_id=oid,
            tenant_required=True,
        )

    def get_workforce_rollout_status(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return self._request("GET", f"/v1/orgs/{oid}/workforce/rollout/status", org_id=oid, tenant_required=True)

    async def get_workforce_rollout_status_async(self, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await self._request_async(
            "GET",
            f"/v1/orgs/{oid}/workforce/rollout/status",
            org_id=oid,
            tenant_required=True,
        )



from __future__ import annotations

from sentinos import LLMGuard, SentinosClient, create_openresponses_adapter

client = SentinosClient.from_env(org_id="acme")
guard = LLMGuard(kernel=client.kernel, agent_id="assistant-1", session_id="sess-openresponses-stream")
adapter = create_openresponses_adapter(guard=guard, client=openai_client)

streamed = adapter.stream(
    model="gpt-4.1-mini",
    input=[{"type": "message", "role": "user", "content": "Draft a postmortem update."}],
    text={"verbosity": "medium"},
    reasoning={"effort": "medium", "summary": "auto"},
)

for event in streamed.events:
    print(event.type)

if streamed.final_response is not None:
    print(
        "final:",
        streamed.final_response.id,
        streamed.final_response.status,
        len(streamed.final_response.output or []),
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from sentinos_core import AuthenticatedClient, Client

from ._http import request_json, request_json_async, require_tenant


@dataclass
class AuditClient:
    _core: Client | AuthenticatedClient
    tenant_id: str | None = None

    def _require_org(self, org_id: str | None) -> str:
        return require_tenant(self.tenant_id, org_id, field_name="org_id")

    def list_events(
        self,
        *,
        org_id: str | None = None,
        from_: str | None = None,
        to: str | None = None,
        q: str | None = None,
        category: str | None = None,
        action: str | None = None,
        actor: str | None = None,
        resource_type: str | None = None,
        outcome: str | None = None,
        source_service: str | None = None,
        tags: str | list[str] | None = None,
        limit: int | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        params: dict[str, Any] = {
            "from": from_,
            "to": to,
            "q": q,
            "category": category,
            "action": action,
            "actor": actor,
            "resource_type": resource_type,
            "outcome": outcome,
            "source_service": source_service,
            "limit": limit,
            "cursor": cursor,
        }
        if isinstance(tags, list):
            params["tags"] = ",".join(tags)
        else:
            params["tags"] = tags
        return request_json(self._core, "GET", f"/v1/orgs/{oid}/audit/events", params=params)

    async def list_events_async(self, **kwargs: Any) -> dict[str, Any]:
        oid = self._require_org(kwargs.pop("org_id", None))
        tags = kwargs.pop("tags", None)
        params = dict(kwargs)
        if isinstance(tags, list):
            params["tags"] = ",".join(tags)
        elif tags is not None:
            params["tags"] = tags
        return await request_json_async(self._core, "GET", f"/v1/orgs/{oid}/audit/events", params=params)

    def get_event(self, event_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "GET", f"/v1/orgs/{oid}/audit/events/{event_id}")

    async def get_event_async(self, event_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "GET", f"/v1/orgs/{oid}/audit/events/{event_id}")

    def export_events(self, *, org_id: str | None = None, export: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "POST", f"/v1/orgs/{oid}/audit/events/export", body=export)

    async def export_events_async(self, *, org_id: str | None = None, export: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "POST", f"/v1/orgs/{oid}/audit/events/export", body=export)

    def list_saved_views(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = request_json(self._core, "GET", f"/v1/orgs/{oid}/audit/saved-views")
        return list(body.get("views") or body.get("saved_views") or [])

    async def list_saved_views_async(self, *, org_id: str | None = None) -> list[dict[str, Any]]:
        oid = self._require_org(org_id)
        body = await request_json_async(self._core, "GET", f"/v1/orgs/{oid}/audit/saved-views")
        return list(body.get("views") or body.get("saved_views") or [])

    def create_saved_view(self, *, org_id: str | None = None, view: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "POST", f"/v1/orgs/{oid}/audit/saved-views", body=view)

    async def create_saved_view_async(self, *, org_id: str | None = None, view: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "POST", f"/v1/orgs/{oid}/audit/saved-views", body=view)

    def update_saved_view(self, view_id: str, *, org_id: str | None = None, view: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "PATCH", f"/v1/orgs/{oid}/audit/saved-views/{view_id}", body=view)

    async def update_saved_view_async(
        self,
        view_id: str,
        *,
        org_id: str | None = None,
        view: dict[str, Any],
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "PATCH", f"/v1/orgs/{oid}/audit/saved-views/{view_id}", body=view)

    def delete_saved_view(self, view_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "DELETE", f"/v1/orgs/{oid}/audit/saved-views/{view_id}")

    async def delete_saved_view_async(self, view_id: str, *, org_id: str | None = None) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "DELETE", f"/v1/orgs/{oid}/audit/saved-views/{view_id}")

    def create_notable_rule(self, *, org_id: str | None = None, rule: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(self._core, "POST", f"/v1/orgs/{oid}/audit/notable-rules", body=rule)

    async def create_notable_rule_async(self, *, org_id: str | None = None, rule: dict[str, Any]) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(self._core, "POST", f"/v1/orgs/{oid}/audit/notable-rules", body=rule)

    def list_notable_events(
        self,
        *,
        org_id: str | None = None,
        limit: int | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return request_json(
            self._core,
            "GET",
            f"/v1/orgs/{oid}/audit/notable-events",
            params={"limit": limit, "cursor": cursor},
        )

    async def list_notable_events_async(
        self,
        *,
        org_id: str | None = None,
        limit: int | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        oid = self._require_org(org_id)
        return await request_json_async(
            self._core,
            "GET",
            f"/v1/orgs/{oid}/audit/notable-events",
            params={"limit": limit, "cursor": cursor},
        )



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from sentinos_core import AuthenticatedClient, Client

from .models.marketplace import InstallResult, MarketplacePack, PackInstall


@dataclass
class MarketplaceClient:
    _core: Client | AuthenticatedClient
    tenant_id: str | None = None

    def _tenant_or_empty(self, tenant_id: str | None) -> str:
        return (tenant_id or self.tenant_id or "").strip()

    def _core_with_headers(self, *, tenant_id: str | None) -> Client | AuthenticatedClient:
        headers: dict[str, str] = {}
        t = self._tenant_or_empty(tenant_id)
        if t:
            headers["x-tenant-id"] = t
        return self._core.with_headers(headers) if headers else self._core

    def list_packs(
        self,
        *,
        search: str | None = None,
        tags: list[str] | None = None,
        verified_only: bool = False,
        tenant_id: str | None = None,
    ) -> list[MarketplacePack]:
        c = self._core_with_headers(tenant_id=tenant_id)
        params: dict[str, Any] = {}
        if search:
            params["search"] = search
        if tags:
            params["tags"] = tags
        if verified_only:
            params["verified_only"] = "true"
        resp = c.get_httpx_client().get("/v1/marketplace/packs", params=params)
        resp.raise_for_status()
        data = resp.json()
        packs = data.get("packs") or []
        return [MarketplacePack.model_validate(p) for p in packs]

    async def list_packs_async(
        self,
        *,
        search: str | None = None,
        tags: list[str] | None = None,
        verified_only: bool = False,
        tenant_id: str | None = None,
    ) -> list[MarketplacePack]:
        c = self._core_with_headers(tenant_id=tenant_id)
        params: dict[str, Any] = {}
        if search:
            params["search"] = search
        if tags:
            params["tags"] = tags
        if verified_only:
            params["verified_only"] = "true"
        resp = await c.get_async_httpx_client().get("/v1/marketplace/packs", params=params)
        resp.raise_for_status()
        data = resp.json()
        packs = data.get("packs") or []
        return [MarketplacePack.model_validate(p) for p in packs]

    def get_pack(self, *, pack_id: str, tenant_id: str | None = None) -> MarketplacePack:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = c.get_httpx_client().get(f"/v1/marketplace/packs/{pack_id}")
        resp.raise_for_status()
        return MarketplacePack.model_validate(resp.json())

    async def get_pack_async(self, *, pack_id: str, tenant_id: str | None = None) -> MarketplacePack:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = await c.get_async_httpx_client().get(f"/v1/marketplace/packs/{pack_id}")
        resp.raise_for_status()
        return MarketplacePack.model_validate(resp.json())

    def install_pack(
        self,
        *,
        pack_id: str,
        target_status: str = "staging",
        skip_simulation: bool = False,
        trace_limit: int = 100,
        tenant_id: str | None = None,
    ) -> InstallResult:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = c.get_httpx_client().post(
            f"/v1/marketplace/packs/{pack_id}/install",
            json={"target_status": target_status, "skip_simulation": skip_simulation, "trace_limit": trace_limit},
        )
        resp.raise_for_status()
        data = resp.json()
        return InstallResult.model_validate(
            {
                "install_id": data.get("install_id"),
                "simulation_job_ids": data.get("simulation_job_ids") or [],
                "raw": data,
            }
        )

    async def install_pack_async(
        self,
        *,
        pack_id: str,
        target_status: str = "staging",
        skip_simulation: bool = False,
        trace_limit: int = 100,
        tenant_id: str | None = None,
    ) -> InstallResult:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = await c.get_async_httpx_client().post(
            f"/v1/marketplace/packs/{pack_id}/install",
            json={"target_status": target_status, "skip_simulation": skip_simulation, "trace_limit": trace_limit},
        )
        resp.raise_for_status()
        data = resp.json()
        return InstallResult.model_validate(
            {
                "install_id": data.get("install_id"),
                "simulation_job_ids": data.get("simulation_job_ids") or [],
                "raw": data,
            }
        )

    def list_installs(self, *, tenant_id: str | None = None) -> list[PackInstall]:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = c.get_httpx_client().get("/v1/marketplace/installs")
        resp.raise_for_status()
        data = resp.json()
        installs = data.get("installs") or []
        return [PackInstall.model_validate(i) for i in installs]

    async def list_installs_async(self, *, tenant_id: str | None = None) -> list[PackInstall]:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = await c.get_async_httpx_client().get("/v1/marketplace/installs")
        resp.raise_for_status()
        data = resp.json()
        installs = data.get("installs") or []
        return [PackInstall.model_validate(i) for i in installs]

    def uninstall_pack(self, *, pack_id: str, tenant_id: str | None = None) -> None:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = c.get_httpx_client().delete(f"/v1/marketplace/installs/{pack_id}")
        resp.raise_for_status()

    async def uninstall_pack_async(self, *, pack_id: str, tenant_id: str | None = None) -> None:
        c = self._core_with_headers(tenant_id=tenant_id)
        resp = await c.get_async_httpx_client().delete(f"/v1/marketplace/installs/{pack_id}")
        resp.raise_for_status()
